#!/usr/bin/python
"""
For http://myzyxel.in.th/
DyDNS server

To update ip by sending a GET query like:
http://www.myzyxel.in.th/update.php?username=ok&pass=123456&ip=127.0.0.1&host=myzyxel.in.th

Note: the value of "ip" is not important, just keyin the correct ID and PW. It will update the actual WAN IP that your device used.

Usage:
#./updateZyxelDDNS.py username password
e.g. #./updateZyxelDDNS.py ok 123456
"""
import httplib
import sys
import re
from threading import Thread, Event
import urllib2

#############################Global Variables#############################
gMyZyXELID = ''
gMyZyXELPW = ''
UPDATE_INTERVAL = 1800 # Update every 30 mins
ZYXEL_DYDNS_LOG = "/tmp/DyDNS.log"

class RepeatableTimer(Thread):
	def __init__(self, interval, function, args=[], kwargs={}):		
		Thread.__init__(self)
		self.interval = interval
		self.function = function
		self.args = args
		self.kwargs = kwargs
		self.event = Event()

	def run(self):     		
		while 1:
			self.event.wait(self.interval)
			self.function(*self.args, **self.kwargs)

# For debug print
def printText(txt):
	lines = txt.split('\n')
	for line in lines:
		print line.strip()

# Go to http://checkip.dyndns.org to get external IP
def getExternalIp():
	ip = -1
	try:
		httpServ = httplib.HTTPConnection("checkip.dyndns.org", 80)
		httpServ.connect()
		httpServ.request('GET', "index.htm")
		response = httpServ.getresponse()
		if response.status == httplib.OK:
			restext = response.read()
			ip = re.findall(r'[0-9]+(?:\.[0-9]+){3}',restext)[0]
		httpServ.close()
	except:
		# Try use urllib2, because there are user reports that sometimes we get httplib.BadStatusLine error when "httpServ.getresponse()"
		# So when we get the BadStatusLine exception, we try another library. This urllib2 library seems work fine.
		# But we don't know the root cause of why we get this exception, maybe some network environment settings in user's LAN. Ticket #8155
		restext = urllib2.urlopen("http://checkip.dyndns.org").read()
		ip = re.findall(r'[0-9]+(?:\.[0-9]+){3}',restext)[0]

	return ip

# The function to send a GET request to update URL link
# "http://www.myzyxel.in.th/update.php?username=ok&pass=123456&ip=127.0.0.1&host=myzyxel.in.th"
def sendUpdateGet():
	fp = open(ZYXEL_DYDNS_LOG, "w")

	# Get external ip first to write LOG.
	ip = getExternalIp()

	httpServ = httplib.HTTPConnection("myzyxel.in.th", 80)
	httpServ.connect()

	getUrlString = "/update.php?username=%s&pass=%s&ip=1.2.3.4&host=myzyxel.in.th" % (gMyZyXELID, gMyZyXELPW)
	httpServ.request('GET', getUrlString)

	response = httpServ.getresponse()
	if response.status == httplib.OK:
		responsetext = response.read()

		# For debug
		print "Output from CGI request:"
		printText (responsetext)

		# Check response. "0" => successful update "1" => invalid login (or other problem) "2" => Domain has expire
		if (int(responsetext) == 0):
			fp.write("SUCCESS\n")
			fp.write(ip+"\n")
		else:
			fp.write("FAILURE\n")
	else:
		fp.write("FAILURE\n")
		
	fp.close()
	httpServ.close()

############################# Main #############################

# Check arguments count
if len(sys.argv) != 3:
	print 'Usage: %s Username Password\ne.g. %s ok 123456' % (sys.argv[0], sys.argv[0])
	sys.exit(-1)

# Set to global variable ID and Password
gMyZyXELID = sys.argv[1]
gMyZyXELPW = sys.argv[2]

# Run first
sendUpdateGet()
# Set a repeat timer to do sendUpdateGet periodically.
rtimer = RepeatableTimer(UPDATE_INTERVAL, sendUpdateGet)
rtimer.start()
